package com.dx.easychat.easychatting.handler;

import com.alibaba.fastjson2.JSON;
import com.alibaba.fastjson2.JSONObject;
import com.baomidou.mybatisplus.core.conditions.query.LambdaQueryWrapper;
import com.dx.easychat.easychatting.entity.FileEntity;
import com.dx.easychat.easychatting.entity.Message;
import com.dx.easychat.easychatting.service.FileEntityService;
import com.dx.easychat.easychatting.service.impl.MessageServiceImpl;
import jakarta.annotation.Resource;
import lombok.extern.log4j.Log4j2;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.BeanUtils;
import org.springframework.stereotype.Component;
import org.springframework.web.socket.*;
import org.springframework.web.socket.handler.TextWebSocketHandler;

import java.io.File;
import java.io.IOException;
import java.net.InetSocketAddress;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

@Log4j2
@Component
public class WebSocketReceiveHandler extends TextWebSocketHandler{

    @Resource
    private MessageServiceImpl messageService;

    @Resource
    private FileEntityService fileEntityService;

    public static ConcurrentHashMap<String, WebSocketSession> SessionsPool = new ConcurrentHashMap<>();

    @Override
    public void afterConnectionEstablished(WebSocketSession session) throws Exception {
        log.info("建立ws链接: id={}", session.getId());
        SessionsPool.put(session.getId(), session);
        JSONObject jsonObject = new JSONObject();
        jsonObject.put("id", session.getId());
        jsonObject.put("messageType", "heartbeat");
        jsonObject.put("status", true);
        jsonObject.put("onlineNumber", SessionsPool.size());
        session.sendMessage(new TextMessage(jsonObject.toJSONString()));
        super.afterConnectionEstablished(session);
    }

    @Override
    protected void handleTextMessage(WebSocketSession session, TextMessage message) throws Exception {
        String payload = message.getPayload();
        InetSocketAddress remoteAddress = session.getRemoteAddress();
        log.info("接收到消息：id={},message={},ip={}", session.getId(), payload, remoteAddress);
        boolean valid = JSON.isValid(payload);
        if (valid) {
            Message messageEntity = JSONObject.parseObject(payload, Message.class);
            log.info("转换实体类：{}", messageEntity);
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("id", session.getId());
            jsonObject.put("messageType", "heartbeat");
            jsonObject.put("status", true);
            jsonObject.put("onlineNumber", SessionsPool.size());
            if (StringUtils.isNoneEmpty(messageEntity.getSessionId()) && SessionsPool.containsKey(session.getId())) {
                switch (messageEntity.getMessageType()) {
                    case "delete":
                        messageService.updateById(messageEntity);
                        if ("file".equals(messageEntity.getType())){
                            LambdaQueryWrapper<FileEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                            lambdaQueryWrapper.eq(FileEntity::getUuid, messageEntity.getFile());
                            FileEntity fileEntity = fileEntityService.getOne(lambdaQueryWrapper);
                            if (fileEntity != null) {
                                File file = new File(fileEntity.getFilePath());
                                if (file.exists()) {
                                    file.delete();
                                }
                                fileEntityService.removeById(fileEntity.getId());
                            }
                        }
                        break;
                    case "heartbeat":
                        session.sendMessage(new TextMessage(jsonObject.toJSONString()));
                        break;
                    case "send":
                        Message messageDto = new Message();
                        BeanUtils.copyProperties(messageEntity, messageDto);
                        messageDto.setIp(Objects.requireNonNull(session.getRemoteAddress()).getAddress().getHostAddress());
                        messageService.save(messageDto);
                        if ("file".equals(messageEntity.getType())){
                            LambdaQueryWrapper<FileEntity> lambdaQueryWrapper = new LambdaQueryWrapper<>();
                            lambdaQueryWrapper.eq(FileEntity::getUuid, messageEntity.getFile());
                            FileEntity fileEntity = fileEntityService.getOne(lambdaQueryWrapper);
                            String fileMimeType = fileEntity.getFileMimeType();
                            messageEntity.setFileMimeType(fileMimeType);
                            messageEntity.setFileName(fileEntity.getFileName());
                        }
                        messageEntity.setMessageType("receive");
                        SessionsPool.forEach((key, value) -> {
                            try {
                                value.sendMessage(new TextMessage(JSONObject.toJSONString(messageEntity)));
                            } catch (IOException e) {
                                log.info("广播消息：{}", e.getMessage());
                            }
                        });
                        break;
                    default:
                        log.error("未知类型数据");
                }
            }else {
                log.error("请求sessionID不存在，重新获取session");
                jsonObject.put("status", false);
                jsonObject.put("message", "请求sessionID不存在，重新下发session");
                session.sendMessage(new TextMessage(jsonObject.toJSONString()));
            }
        }else {
            JSONObject jsonObject = new JSONObject();
            jsonObject.put("code", -1);
            jsonObject.put("message", "请求错误");
            jsonObject.put("time", System.currentTimeMillis());
            session.sendMessage(new TextMessage(jsonObject.toJSONString()));
        }
        super.handleTextMessage(session, message);
    }

    @Override
    protected void handleBinaryMessage(WebSocketSession session, BinaryMessage message) {
        log.info("接收到二进制消息：：id={},message={}", session.getId(), message.getPayload());
        super.handleBinaryMessage(session, message);
    }

    @Override
    public void handleTransportError(WebSocketSession session, Throwable exception) throws Exception {
        log.error("出现异常：id={},message={}", session.getId(), exception.getMessage());
        super.handleTransportError(session, exception);
    }

    @Override
    public void afterConnectionClosed(WebSocketSession session, CloseStatus status) throws Exception {
        String sessionId = session.getId();
        log.info("断开链接：id={},status={}", sessionId, status);
        WebSocketSession remove = SessionsPool.remove(sessionId);
        if (remove != null) {
            remove.close();
        }
        super.afterConnectionClosed(session, status);
    }

}
